import { handleMetadata } from '../../metadata/index.js';
import { themePlain } from './theme.js';
export * from './theme.js';
import { token, tokenSeparator, tokenSpace, tokenTime, tokenText, join, tokenAddr, tokenValue, tokenNetwork, tokenPlatform, tokenAsset, tokenImage, tokenHandle, tokenPost, tokenName } from './token.js';

function getSummaryActions({ actions, tag, type }) {
  if (!actions) return [];
  if (actions.length === 1) return actions;
  const list = ["transaction-multisig"];
  return list.includes(`${tag}-${type}`) ? actions : actions.filter((t) => t.tag === tag && t.type === type);
}
function formatPlain(activity) {
  const list = format(activity, themePlain).filter((s) => s !== "");
  const clean = [];
  for (let i = 0; i < list.length; i++) {
    if (list[i] === " " && list[i + 1] === " ") continue;
    clean.push(list[i] ?? "");
  }
  return clean.join("");
}
function format(activity, theme) {
  const ts = tokenizeActivity(activity);
  return ts.map((t) => {
    if (!theme[t.type]) {
      return theme.unknown(t.content);
    }
    return theme[t.type](t.content);
  });
}
function tokenizeActivity(activity) {
  const actions = getSummaryActions(activity);
  if (activity.tag === "social" && actions.length > 1) {
    return tokenizeAction(activity, actions[0]);
  }
  if (activity.tag === "unknown" || activity.type === "unknown") {
    return [token("unknown", "Carried out an activity")];
  }
  const tokens = actions.reduce((acc, action) => {
    if (acc.length > 0) {
      acc.push(tokenSeparator);
    }
    acc.push(...tokenizeAction(activity, action));
    return acc;
  }, []);
  tokens.push(tokenSpace, tokenTime(activity.timestamp));
  return tokens;
}
function tokenizeToActions(activity) {
  const actions = activity.actions ?? [];
  if (activity.tag === "social" && actions.length > 1) {
    return [tokenizeAction(activity, actions[0])];
  }
  if (activity.tag === "unknown" || activity.type === "unknown") {
    return [[token("unknown", "Carried out an activity")]];
  }
  return actions.map((action) => tokenizeAction(activity, action));
}
function tokenizeToSummaryActions(activity) {
  const actions = getSummaryActions(activity);
  if (activity.tag === "social" && actions.length > 1) {
    return [tokenizeAction(activity, actions[0])];
  }
  if (activity.tag === "unknown" || activity.type === "unknown") {
    return [[token("unknown", "Carried out an activity")]];
  }
  return actions.map((action) => tokenizeAction(activity, action));
}
function tokenizeAction(activity, action) {
  const direction = activity.direction;
  let res = [tokenText("Carried out an activity")];
  handleMetadata(action, {
    "transaction-transfer": (m) => {
      if (direction === "in") {
        res = join([
          tokenAddr(action.to),
          tokenText("received"),
          ...tokenValue(m),
          tokenText("from"),
          tokenAddr(action.from)
        ]);
      } else {
        res = join([
          tokenAddr(action.from),
          tokenText("sent"),
          ...tokenValue(m),
          tokenText("to"),
          tokenAddr(action.to)
        ]);
      }
    },
    "transaction-approval": (m) => {
      if (m.action === "approve") {
        res = join([
          tokenAddr(action.from),
          tokenText("approved"),
          ...tokenValue(m),
          tokenText("to"),
          tokenAddr(action.to)
        ]);
      } else {
        res = join([
          tokenAddr(action.from),
          tokenText("revoked the approval of"),
          ...tokenValue(m),
          tokenText("to"),
          tokenAddr(action.to)
        ]);
      }
    },
    "transaction-mint": (m) => {
      res = join([
        tokenAddr(action.from),
        tokenText("minted"),
        ...tokenValue(m)
      ]);
    },
    "transaction-burn": (m) => {
      res = join([
        tokenAddr(action.from),
        tokenText("burned"),
        ...tokenValue(m)
      ]);
    },
    "transaction-bridge": (m) => {
      let network = [];
      if (m.sourceNetwork) {
        network = [
          tokenText("from"),
          tokenNetwork(m.sourceNetwork),
          tokenText("to"),
          tokenNetwork(m.targetNetwork)
        ];
      }
      if (m.action === "deposit") {
        res = join([
          tokenAddr(action.from),
          tokenText("deposited"),
          ...tokenValue(m.token),
          ...network,
          ...tokenPlatform(activity)
        ]);
      } else {
        res = join([
          tokenAddr(action.from),
          tokenText("withdrew"),
          ...tokenValue(m.token),
          ...network,
          ...tokenPlatform(activity)
        ]);
      }
    },
    // for collectible or nft related action, it will use image_url as the image link
    "collectible-transfer": (m) => {
      const meta = {
        address: m.address,
        id: m.id,
        network: activity.network,
        preview: m.parsedImageUrl
      };
      res = join([
        tokenAddr(action.from),
        tokenText("transferred"),
        ...tokenAsset(m.name || "an asset", meta),
        tokenText("to"),
        tokenAddr(action.to)
      ]);
    },
    "collectible-approval": (m) => {
      const meta = {
        address: m.address,
        id: m.id,
        network: activity.network,
        preview: m.parsedImageUrl
      };
      if (m.action === "approve") {
        res = join([
          tokenAddr(action.from),
          tokenText("approved"),
          tokenImage(m.parsedImageUrl),
          ...tokenAsset(m.name || "collection", meta),
          tokenText("to"),
          tokenAddr(action.to)
        ]);
      } else {
        res = join([
          tokenAddr(action.from),
          tokenText("revoked the approval of"),
          ...tokenAsset(m.name || "collection", meta),
          tokenText("to"),
          tokenAddr(action.to)
        ]);
      }
    },
    "collectible-mint": (m) => {
      const meta = {
        address: m.address,
        id: m.id,
        network: activity.network,
        preview: m.parsedImageUrl
      };
      res = join([
        tokenAddr(action.from),
        tokenText("minted"),
        ...tokenAsset(m.name || "an asset", meta),
        tokenText("to"),
        tokenAddr(action.to)
      ]);
    },
    "collectible-burn": (m) => {
      const meta = {
        address: m.address,
        id: m.id,
        network: activity.network,
        preview: m.parsedImageUrl
      };
      res = join([
        tokenAddr(action.from),
        tokenText("burned"),
        ...tokenAsset(m.name || "an asset", meta)
      ]);
    },
    "collectible-trade": (m) => {
      const meta = {
        address: m.address,
        id: m.id,
        network: activity.network,
        preview: m.parsedImageUrl
      };
      if (m.action === "buy") {
        res = join([
          tokenAddr(action.to),
          tokenText("bought"),
          ...tokenAsset(m.name || "an asset", meta),
          tokenText("from"),
          tokenAddr(action.from),
          ...tokenPlatform(action)
        ]);
      } else {
        res = join([
          tokenAddr(action.from),
          tokenText("sold"),
          ...tokenAsset(m.name || "an asset", meta),
          tokenText("to"),
          tokenAddr(action.to),
          ...tokenPlatform(action)
        ]);
      }
    },
    "exchange-swap": (m) => {
      res = join([
        tokenAddr(action.from),
        tokenText("swapped"),
        ...tokenValue(m.from),
        tokenText("to"),
        ...tokenValue(m.to),
        ...tokenPlatform(action)
      ]);
    },
    // todo add the action invoker
    "exchange-liquidity": (m) => {
      const tokens = m.tokens.flatMap((t) => join([...tokenValue(t), tokenText(",")])).slice(0, -1);
      if (m.action === "add") {
        res = join([
          tokenAddr(action.from),
          tokenText("added"),
          ...tokens,
          tokenText("to liquidity"),
          ...tokenPlatform(action)
        ]);
      } else if (m.action === "remove") {
        res = join([
          tokenAddr(action.from),
          tokenText("removed"),
          ...tokens,
          tokenText("from liquidity"),
          ...tokenPlatform(action)
        ]);
      } else if (m.action === "collect") {
        res = join([
          tokenAddr(action.from),
          tokenText("collected"),
          ...tokens,
          tokenText("from liquidity"),
          ...tokenPlatform(action)
        ]);
      } else if (m.action === "borrow") {
        res = join([
          tokenAddr(action.from),
          tokenText("borrowed"),
          ...tokens,
          tokenText("from liquidity"),
          ...tokenPlatform(action)
        ]);
      } else if (m.action === "repay") {
        res = join([
          tokenAddr(action.from),
          tokenText("repaid"),
          ...tokens,
          tokenText("to liquidity"),
          ...tokenPlatform(action)
        ]);
      } else if (m.action === "supply") {
        res = join([
          tokenAddr(action.from),
          tokenText("supplied"),
          ...tokens,
          tokenText("to liquidity"),
          ...tokenPlatform(action)
        ]);
      } else if (m.action === "withdraw") {
        res = join([
          tokenAddr(action.from),
          tokenText("withDrew"),
          ...tokens,
          tokenText("from liquidity"),
          ...tokenPlatform(action)
        ]);
      }
    },
    "social-post": (m) => {
      res = join([
        tokenHandle(m.handle || action.from, action.from, activity.network),
        tokenText("published a post"),
        tokenPost(action),
        ...tokenPlatform(action)
      ]);
    },
    "social-comment": (m) => {
      res = join([
        tokenHandle(m.handle || action.from, action.from, activity.network),
        tokenText("made a comment"),
        tokenPost(action),
        ...tokenPlatform(action)
      ]);
    },
    "social-share": () => {
      res = join([
        tokenAddr(action.from),
        tokenText("shared a post"),
        tokenPost(action),
        ...tokenPlatform(action)
      ]);
    },
    "social-mint": () => {
      res = join([
        tokenAddr(action.from),
        tokenText("minted a post"),
        tokenPost(action),
        ...tokenPlatform(action)
      ]);
    },
    "social-revise": (m) => {
      res = join([
        tokenAddr(m.handle || action.from),
        tokenText("revised a post"),
        tokenPost(action),
        ...tokenPlatform(action)
      ]);
    },
    "social-profile": (m) => {
      if (m.action === "create") {
        res = join([
          tokenAddr(m.handle || action.from),
          tokenText("created a profile"),
          tokenImage(m.imageUri),
          tokenName(m.name || m.handle || ""),
          ...tokenPlatform(action)
        ]);
      } else if (m.action === "update") {
        res = join([
          tokenAddr(m.handle || action.from),
          tokenText("updated a profile"),
          tokenImage(m.imageUri),
          tokenName(m.name || m.handle || ""),
          ...tokenPlatform(action)
        ]);
      } else if (m.action === "renew") {
        res = join([
          tokenAddr(m.handle || action.from),
          tokenText("renewed a profile"),
          tokenImage(m.imageUri),
          tokenName(m.name || m.handle || ""),
          ...tokenPlatform(action)
        ]);
      } else if (m.action === "wrap") {
        res = join([
          tokenAddr(m.handle || action.from),
          tokenText("wrapped a profile"),
          tokenImage(m.imageUri),
          tokenName(m.name || m.handle || ""),
          ...tokenPlatform(action)
        ]);
      } else if (m.action === "unwrap") {
        res = join([
          tokenAddr(m.handle || action.from),
          tokenText("unwrapped a profile"),
          tokenImage(m.imageUri),
          tokenName(m.name || m.handle || ""),
          ...tokenPlatform(action)
        ]);
      }
    },
    "social-delete": () => {
      res = join([
        tokenAddr(action.from),
        tokenText("deleted a post"),
        tokenPost(action),
        ...tokenPlatform(action)
      ]);
    },
    "social-proxy": (m) => {
      if (m.action === "appoint") {
        res = join([
          tokenAddr(action.from),
          tokenText("approved a proxy"),
          ...tokenPlatform(action)
        ]);
      } else {
        res = join([
          tokenAddr(action.from),
          tokenText("removed a proxy"),
          ...tokenPlatform(action)
        ]);
      }
    },
    "metaverse-transfer": (m) => {
      res = join([
        tokenAddr(action.from),
        tokenText("transferred"),
        tokenImage(m.parsedImageUrl),
        tokenName(m.name || "an asset"),
        tokenText("to"),
        tokenAddr(action.to)
      ]);
    },
    "metaverse-mint": (m) => {
      res = join([
        tokenAddr(action.from),
        tokenText("minted"),
        tokenImage(m.parsedImageUrl),
        tokenName(m.name || "an asset"),
        ...tokenPlatform(action)
      ]);
    },
    "metaverse-burn": (m) => {
      res = join([
        tokenAddr(action.from),
        tokenText("burned"),
        tokenImage(m.parsedImageUrl),
        tokenName(m.name || "an asset"),
        ...tokenPlatform(action)
      ]);
    },
    "metaverse-trade": (m) => {
      if (m.action === "buy") {
        res = join([
          tokenAddr(action.from),
          tokenText("bought"),
          tokenImage(m.parsedImageUrl),
          tokenName(m.name || "an asset"),
          tokenText("from"),
          tokenAddr(action.from),
          ...tokenPlatform(action)
        ]);
      } else if (m.action === "sell") {
        res = join([
          tokenAddr(action.from),
          tokenText("sold"),
          tokenImage(m.parsedImageUrl),
          tokenName(m.name || "an asset"),
          tokenText("from"),
          tokenAddr(action.from),
          ...tokenPlatform(action)
        ]);
      } else if (m.action === "list") {
        res = join([
          tokenAddr(action.from),
          tokenText("listed"),
          tokenImage(m.parsedImageUrl),
          tokenName(m.name || "an asset"),
          tokenText("from"),
          tokenAddr(action.from),
          ...tokenPlatform(action)
        ]);
      }
    }
  });
  return res;
}
function hasMultiPrimaryActions(activity) {
  const actions = getSummaryActions(activity);
  let count = 0;
  for (const action of actions) {
    if (action.type === activity.type && action.tag === activity.tag) {
      count++;
      if (count > 1) {
        return true;
      }
    }
  }
  return false;
}
function flatActivity(activity) {
  const hasMulti = hasMultiPrimaryActions(activity);
  if (hasMulti) {
    return getSummaryActions(activity).map(
      (action) => ({
        ...activity,
        actions: [action]
      })
    );
  }
  return [activity];
}

export { flatActivity, format, formatPlain, getSummaryActions, hasMultiPrimaryActions, tokenizeAction, tokenizeActivity, tokenizeToActions, tokenizeToSummaryActions };
//# sourceMappingURL=index.js.map
//# sourceMappingURL=index.js.map