import { z } from 'zod';
export type OpenApiObject = z.infer<typeof OpenApiObjectSchema>;
/**
 * OpenAPI Object
 *
 * This is the root object of the OpenAPI Description.
 *
 * @see https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.1.1.md#openapi-object
 */
export declare const OpenApiObjectSchema: z.ZodObject<{
    /**
     * REQUIRED. This string MUST be the version number of the OpenAPI Specification that the OpenAPI Document uses. The
     * openapi field SHOULD be used by tooling to interpret the OpenAPI Document. This is not related to the API
     * info.version string.
     */
    openapi: z.ZodString;
    /**
     * REQUIRED. Provides metadata about the API. The metadata MAY be used by tooling as required.
     */
    info: z.ZodObject<{
        title: z.ZodCatch<z.ZodString>;
        summary: z.ZodCatch<z.ZodOptional<z.ZodString>>;
        description: z.ZodCatch<z.ZodOptional<z.ZodString>>;
        termsOfService: z.ZodCatch<z.ZodOptional<z.ZodString>>;
        contact: z.ZodCatch<z.ZodOptional<z.ZodObject<{
            name: z.ZodOptional<z.ZodString>;
            url: z.ZodCatch<z.ZodOptional<z.ZodString>>;
            email: z.ZodCatch<z.ZodOptional<z.ZodString>>;
        }, "strip", z.ZodTypeAny, {
            url?: string | undefined;
            name?: string | undefined;
            email?: string | undefined;
        }, {
            url?: unknown;
            name?: string | undefined;
            email?: unknown;
        }>>>;
        license: z.ZodCatch<z.ZodOptional<z.ZodObject<{
            name: z.ZodCatch<z.ZodNullable<z.ZodOptional<z.ZodString>>>;
            identifier: z.ZodCatch<z.ZodOptional<z.ZodString>>;
            url: z.ZodCatch<z.ZodOptional<z.ZodString>>;
        }, "strip", z.ZodTypeAny, {
            url?: string | undefined;
            name?: string | null | undefined;
            identifier?: string | undefined;
        }, {
            url?: unknown;
            name?: unknown;
            identifier?: unknown;
        }>>>;
        version: z.ZodCatch<z.ZodString>;
    }, "strip", z.ZodTypeAny, {
        title: string;
        version: string;
        description?: string | undefined;
        termsOfService?: string | undefined;
        contact?: {
            url?: string | undefined;
            name?: string | undefined;
            email?: string | undefined;
        } | undefined;
        license?: {
            url?: string | undefined;
            name?: string | null | undefined;
            identifier?: string | undefined;
        } | undefined;
        summary?: string | undefined;
    }, {
        title?: unknown;
        description?: unknown;
        termsOfService?: unknown;
        contact?: unknown;
        license?: unknown;
        version?: unknown;
        summary?: unknown;
    }>;
    /**
     * The default value for the $schema keyword within Schema Objects contained within this OAS document. This MUST be
     * in the form of a URI.
     */
    jsonSchemaDialect: z.ZodOptional<z.ZodString>;
    /**
     * An array of Server Objects, which provide connectivity information to a target server. If the servers field is
     * not provided, or is an empty array, the default value would be a Server Object with a url value of /.
     */
    servers: z.ZodOptional<z.ZodArray<z.ZodObject<{
        url: z.ZodString;
        description: z.ZodOptional<z.ZodString>;
        variables: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
            enum: z.ZodOptional<z.ZodArray<z.ZodString, "many">>;
            default: z.ZodOptional<z.ZodString>;
            description: z.ZodOptional<z.ZodString>;
        }, "strip", z.ZodTypeAny, {
            description?: string | undefined;
            default?: string | undefined;
            enum?: string[] | undefined;
        }, {
            description?: string | undefined;
            default?: string | undefined;
            enum?: string[] | undefined;
        }>>>;
    }, "strip", z.ZodTypeAny, {
        url: string;
        description?: string | undefined;
        variables?: Record<string, {
            description?: string | undefined;
            default?: string | undefined;
            enum?: string[] | undefined;
        }> | undefined;
    }, {
        url: string;
        description?: string | undefined;
        variables?: Record<string, {
            description?: string | undefined;
            default?: string | undefined;
            enum?: string[] | undefined;
        }> | undefined;
    }>, "many">>;
    /**
     * The available paths and operations for the API.
     */
    paths: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<z.objectUtil.extendShape<{
        summary: z.ZodOptional<z.ZodString>;
        description: z.ZodOptional<z.ZodString>;
        servers: z.ZodOptional<z.ZodArray<z.ZodObject<{
            url: z.ZodString;
            description: z.ZodOptional<z.ZodString>;
            variables: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                enum: z.ZodOptional<z.ZodArray<z.ZodString, "many">>;
                default: z.ZodOptional<z.ZodString>;
                description: z.ZodOptional<z.ZodString>;
            }, "strip", z.ZodTypeAny, {
                description?: string | undefined;
                default?: string | undefined;
                enum?: string[] | undefined;
            }, {
                description?: string | undefined;
                default?: string | undefined;
                enum?: string[] | undefined;
            }>>>;
        }, "strip", z.ZodTypeAny, {
            url: string;
            description?: string | undefined;
            variables?: Record<string, {
                description?: string | undefined;
                default?: string | undefined;
                enum?: string[] | undefined;
            }> | undefined;
        }, {
            url: string;
            description?: string | undefined;
            variables?: Record<string, {
                description?: string | undefined;
                default?: string | undefined;
                enum?: string[] | undefined;
            }> | undefined;
        }>, "many">>;
        parameters: z.ZodOptional<z.ZodArray<z.ZodObject<{
            name: z.ZodString;
            in: z.ZodEnum<["query", "header", "path", "cookie"]>;
            description: z.ZodOptional<z.ZodString>;
            required: z.ZodOptional<z.ZodBoolean>;
            deprecated: z.ZodOptional<z.ZodBoolean>;
            allowEmptyValue: z.ZodOptional<z.ZodBoolean>;
            style: z.ZodOptional<z.ZodEnum<["matrix", "label", "form", "simple", "spaceDelimited", "pipeDelimited", "deepObject"]>>;
            explode: z.ZodOptional<z.ZodBoolean>;
            allowReserved: z.ZodOptional<z.ZodBoolean>;
            schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
            example: z.ZodOptional<z.ZodAny>;
            examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                summary: z.ZodOptional<z.ZodString>;
                description: z.ZodOptional<z.ZodString>;
                value: z.ZodOptional<z.ZodAny>;
                externalValue: z.ZodOptional<z.ZodString>;
            }, "strip", z.ZodTypeAny, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }>>>;
            content: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                example: z.ZodOptional<z.ZodAny>;
                examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                    summary: z.ZodOptional<z.ZodString>;
                    description: z.ZodOptional<z.ZodString>;
                    value: z.ZodOptional<z.ZodAny>;
                    externalValue: z.ZodOptional<z.ZodString>;
                }, "strip", z.ZodTypeAny, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }>>>;
                encoding: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                    contentType: z.ZodString;
                    headers: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                        description: z.ZodOptional<z.ZodString>;
                        required: z.ZodOptional<z.ZodBoolean>;
                        deprecated: z.ZodOptional<z.ZodBoolean>;
                        style: z.ZodOptional<z.ZodEnum<["matrix", "label", "simple", "form", "spaceDelimited", "pipeDelimited", "deepObject"]>>;
                        explode: z.ZodOptional<z.ZodBoolean>;
                        schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                        example: z.ZodOptional<z.ZodAny>;
                        examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                            summary: z.ZodOptional<z.ZodString>;
                            description: z.ZodOptional<z.ZodString>;
                            value: z.ZodOptional<z.ZodAny>;
                            externalValue: z.ZodOptional<z.ZodString>;
                        }, "strip", z.ZodTypeAny, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }>>>;
                        content: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                            schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                            example: z.ZodOptional<z.ZodAny>;
                            examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                                summary: z.ZodOptional<z.ZodString>;
                                description: z.ZodOptional<z.ZodString>;
                                value: z.ZodOptional<z.ZodAny>;
                                externalValue: z.ZodOptional<z.ZodString>;
                            }, "strip", z.ZodTypeAny, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }>>>;
                        }, "strip", z.ZodTypeAny, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }>>>;
                    }, "strip", z.ZodTypeAny, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }>>>;
                }, "strip", z.ZodTypeAny, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }>>>;
            }, "strip", z.ZodTypeAny, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }>>>;
        }, "strip", z.ZodTypeAny, {
            name: string;
            in: "path" | "query" | "header" | "cookie";
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
            allowEmptyValue?: boolean | undefined;
            allowReserved?: boolean | undefined;
        }, {
            name: string;
            in: "path" | "query" | "header" | "cookie";
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
            allowEmptyValue?: boolean | undefined;
            allowReserved?: boolean | undefined;
        }>, "many">>;
    }, {
        get: z.ZodOptional<z.ZodType<{
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }, z.ZodTypeDef, {
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }>>;
        put: z.ZodOptional<z.ZodType<{
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }, z.ZodTypeDef, {
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }>>;
        post: z.ZodOptional<z.ZodType<{
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }, z.ZodTypeDef, {
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }>>;
        delete: z.ZodOptional<z.ZodType<{
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }, z.ZodTypeDef, {
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }>>;
        options: z.ZodOptional<z.ZodType<{
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }, z.ZodTypeDef, {
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }>>;
        head: z.ZodOptional<z.ZodType<{
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }, z.ZodTypeDef, {
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }>>;
        patch: z.ZodOptional<z.ZodType<{
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }, z.ZodTypeDef, {
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }>>;
        trace: z.ZodOptional<z.ZodType<{
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }, z.ZodTypeDef, {
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }>>;
    }>, "strip", z.ZodTypeAny, {
        servers?: {
            url: string;
            description?: string | undefined;
            variables?: Record<string, {
                description?: string | undefined;
                default?: string | undefined;
                enum?: string[] | undefined;
            }> | undefined;
        }[] | undefined;
        description?: string | undefined;
        summary?: string | undefined;
        parameters?: {
            name: string;
            in: "path" | "query" | "header" | "cookie";
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
            allowEmptyValue?: boolean | undefined;
            allowReserved?: boolean | undefined;
        }[] | undefined;
        get?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        put?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        post?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        delete?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        options?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        head?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        patch?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        trace?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
    }, {
        servers?: {
            url: string;
            description?: string | undefined;
            variables?: Record<string, {
                description?: string | undefined;
                default?: string | undefined;
                enum?: string[] | undefined;
            }> | undefined;
        }[] | undefined;
        description?: string | undefined;
        summary?: string | undefined;
        parameters?: {
            name: string;
            in: "path" | "query" | "header" | "cookie";
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
            allowEmptyValue?: boolean | undefined;
            allowReserved?: boolean | undefined;
        }[] | undefined;
        get?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        put?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        post?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        delete?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        options?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        head?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        patch?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        trace?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
    }>>>;
    /**
     * The incoming webhooks that MAY be received as part of this API and that the API consumer MAY choose to implement.
     * Closely related to the callbacks feature, this section describes requests initiated other than by an API call,
     * for example by an out of band registration. The key name is a unique string to refer to each webhook, while the
     * (optionally referenced) Path Item Object describes a request that may be initiated by the API provider and the
     * expected responses. An example is available.
     */
    webhooks: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodType<{
        servers?: {
            url: string;
            description?: string | undefined;
            variables?: Record<string, {
                description?: string | undefined;
                default?: string | undefined;
                enum?: string[] | undefined;
            }> | undefined;
        }[] | undefined;
        description?: string | undefined;
        summary?: string | undefined;
        parameters?: {
            name: string;
            in: "path" | "query" | "header" | "cookie";
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
            allowEmptyValue?: boolean | undefined;
            allowReserved?: boolean | undefined;
        }[] | undefined;
    } & {
        get?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        put?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        post?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        delete?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        options?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        head?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        patch?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        trace?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
    }, z.ZodTypeDef, {
        servers?: {
            url: string;
            description?: string | undefined;
            variables?: Record<string, {
                description?: string | undefined;
                default?: string | undefined;
                enum?: string[] | undefined;
            }> | undefined;
        }[] | undefined;
        description?: string | undefined;
        summary?: string | undefined;
        parameters?: {
            name: string;
            in: "path" | "query" | "header" | "cookie";
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
            allowEmptyValue?: boolean | undefined;
            allowReserved?: boolean | undefined;
        }[] | undefined;
    } & {
        get?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        put?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        post?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        delete?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        options?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        head?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        patch?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        trace?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
    }>>>;
    /**
     * An element to hold various Objects for the OpenAPI Description.
     */
    components: z.ZodOptional<z.ZodObject<{
        schemas: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>>;
        responses: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
            description: z.ZodString;
            headers: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                description: z.ZodOptional<z.ZodString>;
                required: z.ZodOptional<z.ZodBoolean>;
                deprecated: z.ZodOptional<z.ZodBoolean>;
                style: z.ZodOptional<z.ZodEnum<["matrix", "label", "simple", "form", "spaceDelimited", "pipeDelimited", "deepObject"]>>;
                explode: z.ZodOptional<z.ZodBoolean>;
                schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                example: z.ZodOptional<z.ZodAny>;
                examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                    summary: z.ZodOptional<z.ZodString>;
                    description: z.ZodOptional<z.ZodString>;
                    value: z.ZodOptional<z.ZodAny>;
                    externalValue: z.ZodOptional<z.ZodString>;
                }, "strip", z.ZodTypeAny, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }>>>;
                content: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                    schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                    example: z.ZodOptional<z.ZodAny>;
                    examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                        summary: z.ZodOptional<z.ZodString>;
                        description: z.ZodOptional<z.ZodString>;
                        value: z.ZodOptional<z.ZodAny>;
                        externalValue: z.ZodOptional<z.ZodString>;
                    }, "strip", z.ZodTypeAny, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }>>>;
                }, "strip", z.ZodTypeAny, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                }, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                }>>>;
            }, "strip", z.ZodTypeAny, {
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
            }, {
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
            }>>>;
            content: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                example: z.ZodOptional<z.ZodAny>;
                examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                    summary: z.ZodOptional<z.ZodString>;
                    description: z.ZodOptional<z.ZodString>;
                    value: z.ZodOptional<z.ZodAny>;
                    externalValue: z.ZodOptional<z.ZodString>;
                }, "strip", z.ZodTypeAny, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }>>>;
                encoding: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                    contentType: z.ZodString;
                    headers: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                        description: z.ZodOptional<z.ZodString>;
                        required: z.ZodOptional<z.ZodBoolean>;
                        deprecated: z.ZodOptional<z.ZodBoolean>;
                        style: z.ZodOptional<z.ZodEnum<["matrix", "label", "simple", "form", "spaceDelimited", "pipeDelimited", "deepObject"]>>;
                        explode: z.ZodOptional<z.ZodBoolean>;
                        schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                        example: z.ZodOptional<z.ZodAny>;
                        examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                            summary: z.ZodOptional<z.ZodString>;
                            description: z.ZodOptional<z.ZodString>;
                            value: z.ZodOptional<z.ZodAny>;
                            externalValue: z.ZodOptional<z.ZodString>;
                        }, "strip", z.ZodTypeAny, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }>>>;
                        content: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                            schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                            example: z.ZodOptional<z.ZodAny>;
                            examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                                summary: z.ZodOptional<z.ZodString>;
                                description: z.ZodOptional<z.ZodString>;
                                value: z.ZodOptional<z.ZodAny>;
                                externalValue: z.ZodOptional<z.ZodString>;
                            }, "strip", z.ZodTypeAny, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }>>>;
                        }, "strip", z.ZodTypeAny, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }>>>;
                    }, "strip", z.ZodTypeAny, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }>>>;
                }, "strip", z.ZodTypeAny, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }>>>;
            }, "strip", z.ZodTypeAny, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }>>>;
            links: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                operationRef: z.ZodOptional<z.ZodString>;
                operationId: z.ZodOptional<z.ZodString>;
                parameters: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodEffects<z.ZodString, string, string>>>;
                requestBody: z.ZodOptional<z.ZodEffects<z.ZodString, string, string>>;
                description: z.ZodOptional<z.ZodString>;
                server: z.ZodOptional<z.ZodObject<{
                    url: z.ZodString;
                    description: z.ZodOptional<z.ZodString>;
                    variables: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                        enum: z.ZodOptional<z.ZodArray<z.ZodString, "many">>;
                        default: z.ZodOptional<z.ZodString>;
                        description: z.ZodOptional<z.ZodString>;
                    }, "strip", z.ZodTypeAny, {
                        description?: string | undefined;
                        default?: string | undefined;
                        enum?: string[] | undefined;
                    }, {
                        description?: string | undefined;
                        default?: string | undefined;
                        enum?: string[] | undefined;
                    }>>>;
                }, "strip", z.ZodTypeAny, {
                    url: string;
                    description?: string | undefined;
                    variables?: Record<string, {
                        description?: string | undefined;
                        default?: string | undefined;
                        enum?: string[] | undefined;
                    }> | undefined;
                }, {
                    url: string;
                    description?: string | undefined;
                    variables?: Record<string, {
                        description?: string | undefined;
                        default?: string | undefined;
                        enum?: string[] | undefined;
                    }> | undefined;
                }>>;
            }, "strip", z.ZodTypeAny, {
                description?: string | undefined;
                operationId?: string | undefined;
                parameters?: Record<string, string> | undefined;
                requestBody?: string | undefined;
                operationRef?: string | undefined;
                server?: {
                    url: string;
                    description?: string | undefined;
                    variables?: Record<string, {
                        description?: string | undefined;
                        default?: string | undefined;
                        enum?: string[] | undefined;
                    }> | undefined;
                } | undefined;
            }, {
                description?: string | undefined;
                operationId?: string | undefined;
                parameters?: Record<string, string> | undefined;
                requestBody?: string | undefined;
                operationRef?: string | undefined;
                server?: {
                    url: string;
                    description?: string | undefined;
                    variables?: Record<string, {
                        description?: string | undefined;
                        default?: string | undefined;
                        enum?: string[] | undefined;
                    }> | undefined;
                } | undefined;
            }>>>;
        }, "strip", z.ZodTypeAny, {
            description: string;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            headers?: Record<string, {
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
            }> | undefined;
            links?: Record<string, {
                description?: string | undefined;
                operationId?: string | undefined;
                parameters?: Record<string, string> | undefined;
                requestBody?: string | undefined;
                operationRef?: string | undefined;
                server?: {
                    url: string;
                    description?: string | undefined;
                    variables?: Record<string, {
                        description?: string | undefined;
                        default?: string | undefined;
                        enum?: string[] | undefined;
                    }> | undefined;
                } | undefined;
            }> | undefined;
        }, {
            description: string;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            headers?: Record<string, {
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
            }> | undefined;
            links?: Record<string, {
                description?: string | undefined;
                operationId?: string | undefined;
                parameters?: Record<string, string> | undefined;
                requestBody?: string | undefined;
                operationRef?: string | undefined;
                server?: {
                    url: string;
                    description?: string | undefined;
                    variables?: Record<string, {
                        description?: string | undefined;
                        default?: string | undefined;
                        enum?: string[] | undefined;
                    }> | undefined;
                } | undefined;
            }> | undefined;
        }>>>;
        parameters: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
            name: z.ZodString;
            in: z.ZodEnum<["query", "header", "path", "cookie"]>;
            description: z.ZodOptional<z.ZodString>;
            required: z.ZodOptional<z.ZodBoolean>;
            deprecated: z.ZodOptional<z.ZodBoolean>;
            allowEmptyValue: z.ZodOptional<z.ZodBoolean>;
            style: z.ZodOptional<z.ZodEnum<["matrix", "label", "form", "simple", "spaceDelimited", "pipeDelimited", "deepObject"]>>;
            explode: z.ZodOptional<z.ZodBoolean>;
            allowReserved: z.ZodOptional<z.ZodBoolean>;
            schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
            example: z.ZodOptional<z.ZodAny>;
            examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                summary: z.ZodOptional<z.ZodString>;
                description: z.ZodOptional<z.ZodString>;
                value: z.ZodOptional<z.ZodAny>;
                externalValue: z.ZodOptional<z.ZodString>;
            }, "strip", z.ZodTypeAny, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }>>>;
            content: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                example: z.ZodOptional<z.ZodAny>;
                examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                    summary: z.ZodOptional<z.ZodString>;
                    description: z.ZodOptional<z.ZodString>;
                    value: z.ZodOptional<z.ZodAny>;
                    externalValue: z.ZodOptional<z.ZodString>;
                }, "strip", z.ZodTypeAny, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }>>>;
                encoding: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                    contentType: z.ZodString;
                    headers: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                        description: z.ZodOptional<z.ZodString>;
                        required: z.ZodOptional<z.ZodBoolean>;
                        deprecated: z.ZodOptional<z.ZodBoolean>;
                        style: z.ZodOptional<z.ZodEnum<["matrix", "label", "simple", "form", "spaceDelimited", "pipeDelimited", "deepObject"]>>;
                        explode: z.ZodOptional<z.ZodBoolean>;
                        schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                        example: z.ZodOptional<z.ZodAny>;
                        examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                            summary: z.ZodOptional<z.ZodString>;
                            description: z.ZodOptional<z.ZodString>;
                            value: z.ZodOptional<z.ZodAny>;
                            externalValue: z.ZodOptional<z.ZodString>;
                        }, "strip", z.ZodTypeAny, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }>>>;
                        content: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                            schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                            example: z.ZodOptional<z.ZodAny>;
                            examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                                summary: z.ZodOptional<z.ZodString>;
                                description: z.ZodOptional<z.ZodString>;
                                value: z.ZodOptional<z.ZodAny>;
                                externalValue: z.ZodOptional<z.ZodString>;
                            }, "strip", z.ZodTypeAny, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }>>>;
                        }, "strip", z.ZodTypeAny, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }>>>;
                    }, "strip", z.ZodTypeAny, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }>>>;
                }, "strip", z.ZodTypeAny, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }>>>;
            }, "strip", z.ZodTypeAny, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }>>>;
        }, "strip", z.ZodTypeAny, {
            name: string;
            in: "path" | "query" | "header" | "cookie";
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
            allowEmptyValue?: boolean | undefined;
            allowReserved?: boolean | undefined;
        }, {
            name: string;
            in: "path" | "query" | "header" | "cookie";
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
            allowEmptyValue?: boolean | undefined;
            allowReserved?: boolean | undefined;
        }>>>;
        examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
            summary: z.ZodOptional<z.ZodString>;
            description: z.ZodOptional<z.ZodString>;
            value: z.ZodOptional<z.ZodAny>;
            externalValue: z.ZodOptional<z.ZodString>;
        }, "strip", z.ZodTypeAny, {
            description?: string | undefined;
            summary?: string | undefined;
            value?: any;
            externalValue?: string | undefined;
        }, {
            description?: string | undefined;
            summary?: string | undefined;
            value?: any;
            externalValue?: string | undefined;
        }>>>;
        requestBodies: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
            description: z.ZodOptional<z.ZodString>;
            content: z.ZodRecord<z.ZodString, z.ZodObject<{
                schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                example: z.ZodOptional<z.ZodAny>;
                examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                    summary: z.ZodOptional<z.ZodString>;
                    description: z.ZodOptional<z.ZodString>;
                    value: z.ZodOptional<z.ZodAny>;
                    externalValue: z.ZodOptional<z.ZodString>;
                }, "strip", z.ZodTypeAny, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }>>>;
                encoding: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                    contentType: z.ZodString;
                    headers: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                        description: z.ZodOptional<z.ZodString>;
                        required: z.ZodOptional<z.ZodBoolean>;
                        deprecated: z.ZodOptional<z.ZodBoolean>;
                        style: z.ZodOptional<z.ZodEnum<["matrix", "label", "simple", "form", "spaceDelimited", "pipeDelimited", "deepObject"]>>;
                        explode: z.ZodOptional<z.ZodBoolean>;
                        schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                        example: z.ZodOptional<z.ZodAny>;
                        examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                            summary: z.ZodOptional<z.ZodString>;
                            description: z.ZodOptional<z.ZodString>;
                            value: z.ZodOptional<z.ZodAny>;
                            externalValue: z.ZodOptional<z.ZodString>;
                        }, "strip", z.ZodTypeAny, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }>>>;
                        content: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                            schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                            example: z.ZodOptional<z.ZodAny>;
                            examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                                summary: z.ZodOptional<z.ZodString>;
                                description: z.ZodOptional<z.ZodString>;
                                value: z.ZodOptional<z.ZodAny>;
                                externalValue: z.ZodOptional<z.ZodString>;
                            }, "strip", z.ZodTypeAny, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }>>>;
                        }, "strip", z.ZodTypeAny, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }>>>;
                    }, "strip", z.ZodTypeAny, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }>>>;
                }, "strip", z.ZodTypeAny, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }>>>;
            }, "strip", z.ZodTypeAny, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }>>;
            required: z.ZodOptional<z.ZodBoolean>;
            encoding: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                contentType: z.ZodString;
                headers: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                    description: z.ZodOptional<z.ZodString>;
                    required: z.ZodOptional<z.ZodBoolean>;
                    deprecated: z.ZodOptional<z.ZodBoolean>;
                    style: z.ZodOptional<z.ZodEnum<["matrix", "label", "simple", "form", "spaceDelimited", "pipeDelimited", "deepObject"]>>;
                    explode: z.ZodOptional<z.ZodBoolean>;
                    schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                    example: z.ZodOptional<z.ZodAny>;
                    examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                        summary: z.ZodOptional<z.ZodString>;
                        description: z.ZodOptional<z.ZodString>;
                        value: z.ZodOptional<z.ZodAny>;
                        externalValue: z.ZodOptional<z.ZodString>;
                    }, "strip", z.ZodTypeAny, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }>>>;
                    content: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                        schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                        example: z.ZodOptional<z.ZodAny>;
                        examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                            summary: z.ZodOptional<z.ZodString>;
                            description: z.ZodOptional<z.ZodString>;
                            value: z.ZodOptional<z.ZodAny>;
                            externalValue: z.ZodOptional<z.ZodString>;
                        }, "strip", z.ZodTypeAny, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }>>>;
                    }, "strip", z.ZodTypeAny, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }>>>;
                }, "strip", z.ZodTypeAny, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }>>>;
            }, "strip", z.ZodTypeAny, {
                contentType: string;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
            }, {
                contentType: string;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
            }>>>;
        }, "strip", z.ZodTypeAny, {
            content: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }>;
            description?: string | undefined;
            required?: boolean | undefined;
            encoding?: Record<string, {
                contentType: string;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
            }> | undefined;
        }, {
            content: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }>;
            description?: string | undefined;
            required?: boolean | undefined;
            encoding?: Record<string, {
                contentType: string;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
            }> | undefined;
        }>>>;
        headers: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
            description: z.ZodOptional<z.ZodString>;
            required: z.ZodOptional<z.ZodBoolean>;
            deprecated: z.ZodOptional<z.ZodBoolean>;
            style: z.ZodOptional<z.ZodEnum<["matrix", "label", "simple", "form", "spaceDelimited", "pipeDelimited", "deepObject"]>>;
            explode: z.ZodOptional<z.ZodBoolean>;
            schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
            example: z.ZodOptional<z.ZodAny>;
            examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                summary: z.ZodOptional<z.ZodString>;
                description: z.ZodOptional<z.ZodString>;
                value: z.ZodOptional<z.ZodAny>;
                externalValue: z.ZodOptional<z.ZodString>;
            }, "strip", z.ZodTypeAny, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }>>>;
            content: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                example: z.ZodOptional<z.ZodAny>;
                examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                    summary: z.ZodOptional<z.ZodString>;
                    description: z.ZodOptional<z.ZodString>;
                    value: z.ZodOptional<z.ZodAny>;
                    externalValue: z.ZodOptional<z.ZodString>;
                }, "strip", z.ZodTypeAny, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }>>>;
            }, "strip", z.ZodTypeAny, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
            }, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
            }>>>;
        }, "strip", z.ZodTypeAny, {
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
        }, {
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
        }>>>;
        securitySchemes: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodUnion<[z.ZodObject<z.objectUtil.extendShape<{
            description: z.ZodOptional<z.ZodString>;
        }, {
            type: z.ZodLiteral<"apiKey">;
            name: z.ZodDefault<z.ZodOptional<z.ZodString>>;
            in: z.ZodCatch<z.ZodDefault<z.ZodOptional<z.ZodEnum<["query", "header", "cookie"]>>>>;
        }>, "strip", z.ZodTypeAny, {
            name: string;
            in: "query" | "header" | "cookie";
            type: "apiKey";
            description?: string | undefined;
        }, {
            type: "apiKey";
            description?: string | undefined;
            name?: string | undefined;
            in?: unknown;
        }>, z.ZodObject<z.objectUtil.extendShape<{
            description: z.ZodOptional<z.ZodString>;
        }, {
            type: z.ZodLiteral<"http">;
            scheme: z.ZodDefault<z.ZodOptional<z.ZodPipeline<z.ZodString, z.ZodEnum<["basic", "bearer"]>>>>;
            bearerFormat: z.ZodOptional<z.ZodUnion<[z.ZodLiteral<"JWT">, z.ZodLiteral<"bearer">, z.ZodString]>>;
        }>, "strip", z.ZodTypeAny, {
            type: "http";
            scheme: "basic" | "bearer";
            description?: string | undefined;
            bearerFormat?: string | undefined;
        }, {
            type: "http";
            description?: string | undefined;
            scheme?: string | undefined;
            bearerFormat?: string | undefined;
        }>, z.ZodObject<z.objectUtil.extendShape<{
            description: z.ZodOptional<z.ZodString>;
        }, {
            type: z.ZodLiteral<"mutualTLS">;
        }>, "strip", z.ZodTypeAny, {
            type: "mutualTLS";
            description?: string | undefined;
        }, {
            type: "mutualTLS";
            description?: string | undefined;
        }>, z.ZodObject<z.objectUtil.extendShape<{
            description: z.ZodOptional<z.ZodString>;
        }, {
            type: z.ZodLiteral<"oauth2">;
            flows: z.ZodObject<{
                implicit: z.ZodOptional<z.ZodOptional<z.ZodObject<z.objectUtil.extendShape<{
                    refreshUrl: z.ZodOptional<z.ZodString>;
                    scopes: z.ZodCatch<z.ZodDefault<z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodOptional<z.ZodString>>>>>;
                }, {
                    type: z.ZodOptional<z.ZodLiteral<"implicit">>;
                    authorizationUrl: z.ZodDefault<z.ZodString>;
                }>, "strip", z.ZodTypeAny, {
                    scopes: Record<string, string | undefined>;
                    authorizationUrl: string;
                    type?: "implicit" | undefined;
                    refreshUrl?: string | undefined;
                }, {
                    type?: "implicit" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    authorizationUrl?: string | undefined;
                }>>>;
                password: z.ZodOptional<z.ZodOptional<z.ZodObject<z.objectUtil.extendShape<{
                    refreshUrl: z.ZodOptional<z.ZodString>;
                    scopes: z.ZodCatch<z.ZodDefault<z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodOptional<z.ZodString>>>>>;
                }, {
                    type: z.ZodOptional<z.ZodLiteral<"password">>;
                    tokenUrl: z.ZodDefault<z.ZodString>;
                }>, "strip", z.ZodTypeAny, {
                    scopes: Record<string, string | undefined>;
                    tokenUrl: string;
                    type?: "password" | undefined;
                    refreshUrl?: string | undefined;
                }, {
                    type?: "password" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    tokenUrl?: string | undefined;
                }>>>;
                clientCredentials: z.ZodOptional<z.ZodOptional<z.ZodObject<z.objectUtil.extendShape<{
                    refreshUrl: z.ZodOptional<z.ZodString>;
                    scopes: z.ZodCatch<z.ZodDefault<z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodOptional<z.ZodString>>>>>;
                }, {
                    type: z.ZodOptional<z.ZodLiteral<"clientCredentials">>;
                    tokenUrl: z.ZodDefault<z.ZodString>;
                }>, "strip", z.ZodTypeAny, {
                    scopes: Record<string, string | undefined>;
                    tokenUrl: string;
                    type?: "clientCredentials" | undefined;
                    refreshUrl?: string | undefined;
                }, {
                    type?: "clientCredentials" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    tokenUrl?: string | undefined;
                }>>>;
                authorizationCode: z.ZodOptional<z.ZodOptional<z.ZodObject<z.objectUtil.extendShape<{
                    refreshUrl: z.ZodOptional<z.ZodString>;
                    scopes: z.ZodCatch<z.ZodDefault<z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodOptional<z.ZodString>>>>>;
                }, {
                    type: z.ZodOptional<z.ZodLiteral<"authorizationCode">>;
                    authorizationUrl: z.ZodDefault<z.ZodString>;
                    tokenUrl: z.ZodDefault<z.ZodString>;
                }>, "strip", z.ZodTypeAny, {
                    scopes: Record<string, string | undefined>;
                    authorizationUrl: string;
                    tokenUrl: string;
                    type?: "authorizationCode" | undefined;
                    refreshUrl?: string | undefined;
                }, {
                    type?: "authorizationCode" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    authorizationUrl?: string | undefined;
                    tokenUrl?: string | undefined;
                }>>>;
            }, "strip", z.ZodTypeAny, {
                implicit?: {
                    scopes: Record<string, string | undefined>;
                    authorizationUrl: string;
                    type?: "implicit" | undefined;
                    refreshUrl?: string | undefined;
                } | undefined;
                password?: {
                    scopes: Record<string, string | undefined>;
                    tokenUrl: string;
                    type?: "password" | undefined;
                    refreshUrl?: string | undefined;
                } | undefined;
                clientCredentials?: {
                    scopes: Record<string, string | undefined>;
                    tokenUrl: string;
                    type?: "clientCredentials" | undefined;
                    refreshUrl?: string | undefined;
                } | undefined;
                authorizationCode?: {
                    scopes: Record<string, string | undefined>;
                    authorizationUrl: string;
                    tokenUrl: string;
                    type?: "authorizationCode" | undefined;
                    refreshUrl?: string | undefined;
                } | undefined;
            }, {
                implicit?: {
                    type?: "implicit" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    authorizationUrl?: string | undefined;
                } | undefined;
                password?: {
                    type?: "password" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    tokenUrl?: string | undefined;
                } | undefined;
                clientCredentials?: {
                    type?: "clientCredentials" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    tokenUrl?: string | undefined;
                } | undefined;
                authorizationCode?: {
                    type?: "authorizationCode" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    authorizationUrl?: string | undefined;
                    tokenUrl?: string | undefined;
                } | undefined;
            }>;
        }>, "strip", z.ZodTypeAny, {
            type: "oauth2";
            flows: {
                implicit?: {
                    scopes: Record<string, string | undefined>;
                    authorizationUrl: string;
                    type?: "implicit" | undefined;
                    refreshUrl?: string | undefined;
                } | undefined;
                password?: {
                    scopes: Record<string, string | undefined>;
                    tokenUrl: string;
                    type?: "password" | undefined;
                    refreshUrl?: string | undefined;
                } | undefined;
                clientCredentials?: {
                    scopes: Record<string, string | undefined>;
                    tokenUrl: string;
                    type?: "clientCredentials" | undefined;
                    refreshUrl?: string | undefined;
                } | undefined;
                authorizationCode?: {
                    scopes: Record<string, string | undefined>;
                    authorizationUrl: string;
                    tokenUrl: string;
                    type?: "authorizationCode" | undefined;
                    refreshUrl?: string | undefined;
                } | undefined;
            };
            description?: string | undefined;
        }, {
            type: "oauth2";
            flows: {
                implicit?: {
                    type?: "implicit" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    authorizationUrl?: string | undefined;
                } | undefined;
                password?: {
                    type?: "password" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    tokenUrl?: string | undefined;
                } | undefined;
                clientCredentials?: {
                    type?: "clientCredentials" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    tokenUrl?: string | undefined;
                } | undefined;
                authorizationCode?: {
                    type?: "authorizationCode" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    authorizationUrl?: string | undefined;
                    tokenUrl?: string | undefined;
                } | undefined;
            };
            description?: string | undefined;
        }>, z.ZodObject<z.objectUtil.extendShape<{
            description: z.ZodOptional<z.ZodString>;
        }, {
            type: z.ZodLiteral<"openIdConnect">;
            openIdConnectUrl: z.ZodDefault<z.ZodOptional<z.ZodString>>;
        }>, "strip", z.ZodTypeAny, {
            type: "openIdConnect";
            openIdConnectUrl: string;
            description?: string | undefined;
        }, {
            type: "openIdConnect";
            description?: string | undefined;
            openIdConnectUrl?: string | undefined;
        }>]>>>;
        links: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
            operationRef: z.ZodOptional<z.ZodString>;
            operationId: z.ZodOptional<z.ZodString>;
            parameters: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodEffects<z.ZodString, string, string>>>;
            requestBody: z.ZodOptional<z.ZodEffects<z.ZodString, string, string>>;
            description: z.ZodOptional<z.ZodString>;
            server: z.ZodOptional<z.ZodObject<{
                url: z.ZodString;
                description: z.ZodOptional<z.ZodString>;
                variables: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                    enum: z.ZodOptional<z.ZodArray<z.ZodString, "many">>;
                    default: z.ZodOptional<z.ZodString>;
                    description: z.ZodOptional<z.ZodString>;
                }, "strip", z.ZodTypeAny, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }>>>;
            }, "strip", z.ZodTypeAny, {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            }, {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            }>>;
        }, "strip", z.ZodTypeAny, {
            description?: string | undefined;
            operationId?: string | undefined;
            parameters?: Record<string, string> | undefined;
            requestBody?: string | undefined;
            operationRef?: string | undefined;
            server?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            } | undefined;
        }, {
            description?: string | undefined;
            operationId?: string | undefined;
            parameters?: Record<string, string> | undefined;
            requestBody?: string | undefined;
            operationRef?: string | undefined;
            server?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            } | undefined;
        }>>>;
        callbacks: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodRecord<z.ZodEffects<z.ZodString, string, string>, z.ZodType<{
            servers?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            }[] | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
        } & {
            get?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            put?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            post?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            delete?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            options?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            head?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            patch?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            trace?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        }, z.ZodTypeDef, {
            servers?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            }[] | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
        } & {
            get?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            put?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            post?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            delete?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            options?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            head?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            patch?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            trace?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        }>>>>;
        pathItems: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<z.objectUtil.extendShape<{
            summary: z.ZodOptional<z.ZodString>;
            description: z.ZodOptional<z.ZodString>;
            servers: z.ZodOptional<z.ZodArray<z.ZodObject<{
                url: z.ZodString;
                description: z.ZodOptional<z.ZodString>;
                variables: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                    enum: z.ZodOptional<z.ZodArray<z.ZodString, "many">>;
                    default: z.ZodOptional<z.ZodString>;
                    description: z.ZodOptional<z.ZodString>;
                }, "strip", z.ZodTypeAny, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }>>>;
            }, "strip", z.ZodTypeAny, {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            }, {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            }>, "many">>;
            parameters: z.ZodOptional<z.ZodArray<z.ZodObject<{
                name: z.ZodString;
                in: z.ZodEnum<["query", "header", "path", "cookie"]>;
                description: z.ZodOptional<z.ZodString>;
                required: z.ZodOptional<z.ZodBoolean>;
                deprecated: z.ZodOptional<z.ZodBoolean>;
                allowEmptyValue: z.ZodOptional<z.ZodBoolean>;
                style: z.ZodOptional<z.ZodEnum<["matrix", "label", "form", "simple", "spaceDelimited", "pipeDelimited", "deepObject"]>>;
                explode: z.ZodOptional<z.ZodBoolean>;
                allowReserved: z.ZodOptional<z.ZodBoolean>;
                schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                example: z.ZodOptional<z.ZodAny>;
                examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                    summary: z.ZodOptional<z.ZodString>;
                    description: z.ZodOptional<z.ZodString>;
                    value: z.ZodOptional<z.ZodAny>;
                    externalValue: z.ZodOptional<z.ZodString>;
                }, "strip", z.ZodTypeAny, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }>>>;
                content: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                    schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                    example: z.ZodOptional<z.ZodAny>;
                    examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                        summary: z.ZodOptional<z.ZodString>;
                        description: z.ZodOptional<z.ZodString>;
                        value: z.ZodOptional<z.ZodAny>;
                        externalValue: z.ZodOptional<z.ZodString>;
                    }, "strip", z.ZodTypeAny, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }>>>;
                    encoding: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                        contentType: z.ZodString;
                        headers: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                            description: z.ZodOptional<z.ZodString>;
                            required: z.ZodOptional<z.ZodBoolean>;
                            deprecated: z.ZodOptional<z.ZodBoolean>;
                            style: z.ZodOptional<z.ZodEnum<["matrix", "label", "simple", "form", "spaceDelimited", "pipeDelimited", "deepObject"]>>;
                            explode: z.ZodOptional<z.ZodBoolean>;
                            schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                            example: z.ZodOptional<z.ZodAny>;
                            examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                                summary: z.ZodOptional<z.ZodString>;
                                description: z.ZodOptional<z.ZodString>;
                                value: z.ZodOptional<z.ZodAny>;
                                externalValue: z.ZodOptional<z.ZodString>;
                            }, "strip", z.ZodTypeAny, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }>>>;
                            content: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                                schema: z.ZodOptional<z.ZodType<Record<string, any>, z.ZodTypeDef, Record<string, any>>>;
                                example: z.ZodOptional<z.ZodAny>;
                                examples: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodObject<{
                                    summary: z.ZodOptional<z.ZodString>;
                                    description: z.ZodOptional<z.ZodString>;
                                    value: z.ZodOptional<z.ZodAny>;
                                    externalValue: z.ZodOptional<z.ZodString>;
                                }, "strip", z.ZodTypeAny, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }>>>;
                            }, "strip", z.ZodTypeAny, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }>>>;
                        }, "strip", z.ZodTypeAny, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }>>>;
                    }, "strip", z.ZodTypeAny, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }>>>;
                }, "strip", z.ZodTypeAny, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>>>;
            }, "strip", z.ZodTypeAny, {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }, {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }>, "many">>;
        }, {
            get: z.ZodOptional<z.ZodType<{
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }, z.ZodTypeDef, {
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }>>;
            put: z.ZodOptional<z.ZodType<{
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }, z.ZodTypeDef, {
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }>>;
            post: z.ZodOptional<z.ZodType<{
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }, z.ZodTypeDef, {
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }>>;
            delete: z.ZodOptional<z.ZodType<{
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }, z.ZodTypeDef, {
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }>>;
            options: z.ZodOptional<z.ZodType<{
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }, z.ZodTypeDef, {
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }>>;
            head: z.ZodOptional<z.ZodType<{
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }, z.ZodTypeDef, {
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }>>;
            patch: z.ZodOptional<z.ZodType<{
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }, z.ZodTypeDef, {
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }>>;
            trace: z.ZodOptional<z.ZodType<{
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }, z.ZodTypeDef, {
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }>>;
        }>, "strip", z.ZodTypeAny, {
            servers?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            }[] | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            get?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            put?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            post?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            delete?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            options?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            head?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            patch?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            trace?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
        }, {
            servers?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            }[] | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            get?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            put?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            post?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            delete?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            options?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            head?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            patch?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            trace?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
        }>>>;
    }, "strip", z.ZodTypeAny, {
        parameters?: Record<string, {
            name: string;
            in: "path" | "query" | "header" | "cookie";
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
            allowEmptyValue?: boolean | undefined;
            allowReserved?: boolean | undefined;
        }> | undefined;
        responses?: Record<string, {
            description: string;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            headers?: Record<string, {
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
            }> | undefined;
            links?: Record<string, {
                description?: string | undefined;
                operationId?: string | undefined;
                parameters?: Record<string, string> | undefined;
                requestBody?: string | undefined;
                operationRef?: string | undefined;
                server?: {
                    url: string;
                    description?: string | undefined;
                    variables?: Record<string, {
                        description?: string | undefined;
                        default?: string | undefined;
                        enum?: string[] | undefined;
                    }> | undefined;
                } | undefined;
            }> | undefined;
        }> | undefined;
        callbacks?: Record<string, Record<string, {
            servers?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            }[] | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
        } & {
            get?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            put?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            post?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            delete?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            options?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            head?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            patch?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            trace?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        }>> | undefined;
        headers?: Record<string, {
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
        }> | undefined;
        links?: Record<string, {
            description?: string | undefined;
            operationId?: string | undefined;
            parameters?: Record<string, string> | undefined;
            requestBody?: string | undefined;
            operationRef?: string | undefined;
            server?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            } | undefined;
        }> | undefined;
        examples?: Record<string, {
            description?: string | undefined;
            summary?: string | undefined;
            value?: any;
            externalValue?: string | undefined;
        }> | undefined;
        schemas?: Record<string, Record<string, any>> | undefined;
        requestBodies?: Record<string, {
            content: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }>;
            description?: string | undefined;
            required?: boolean | undefined;
            encoding?: Record<string, {
                contentType: string;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
            }> | undefined;
        }> | undefined;
        securitySchemes?: Record<string, {
            name: string;
            in: "query" | "header" | "cookie";
            type: "apiKey";
            description?: string | undefined;
        } | {
            type: "http";
            scheme: "basic" | "bearer";
            description?: string | undefined;
            bearerFormat?: string | undefined;
        } | {
            type: "openIdConnect";
            openIdConnectUrl: string;
            description?: string | undefined;
        } | {
            type: "oauth2";
            flows: {
                implicit?: {
                    scopes: Record<string, string | undefined>;
                    authorizationUrl: string;
                    type?: "implicit" | undefined;
                    refreshUrl?: string | undefined;
                } | undefined;
                password?: {
                    scopes: Record<string, string | undefined>;
                    tokenUrl: string;
                    type?: "password" | undefined;
                    refreshUrl?: string | undefined;
                } | undefined;
                clientCredentials?: {
                    scopes: Record<string, string | undefined>;
                    tokenUrl: string;
                    type?: "clientCredentials" | undefined;
                    refreshUrl?: string | undefined;
                } | undefined;
                authorizationCode?: {
                    scopes: Record<string, string | undefined>;
                    authorizationUrl: string;
                    tokenUrl: string;
                    type?: "authorizationCode" | undefined;
                    refreshUrl?: string | undefined;
                } | undefined;
            };
            description?: string | undefined;
        } | {
            type: "mutualTLS";
            description?: string | undefined;
        }> | undefined;
        pathItems?: Record<string, {
            servers?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            }[] | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            get?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            put?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            post?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            delete?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            options?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            head?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            patch?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            trace?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
        }> | undefined;
    }, {
        parameters?: Record<string, {
            name: string;
            in: "path" | "query" | "header" | "cookie";
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
            allowEmptyValue?: boolean | undefined;
            allowReserved?: boolean | undefined;
        }> | undefined;
        responses?: Record<string, {
            description: string;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            headers?: Record<string, {
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
            }> | undefined;
            links?: Record<string, {
                description?: string | undefined;
                operationId?: string | undefined;
                parameters?: Record<string, string> | undefined;
                requestBody?: string | undefined;
                operationRef?: string | undefined;
                server?: {
                    url: string;
                    description?: string | undefined;
                    variables?: Record<string, {
                        description?: string | undefined;
                        default?: string | undefined;
                        enum?: string[] | undefined;
                    }> | undefined;
                } | undefined;
            }> | undefined;
        }> | undefined;
        callbacks?: Record<string, Record<string, {
            servers?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            }[] | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
        } & {
            get?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            put?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            post?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            delete?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            options?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            head?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            patch?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            trace?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        }>> | undefined;
        headers?: Record<string, {
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
        }> | undefined;
        links?: Record<string, {
            description?: string | undefined;
            operationId?: string | undefined;
            parameters?: Record<string, string> | undefined;
            requestBody?: string | undefined;
            operationRef?: string | undefined;
            server?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            } | undefined;
        }> | undefined;
        examples?: Record<string, {
            description?: string | undefined;
            summary?: string | undefined;
            value?: any;
            externalValue?: string | undefined;
        }> | undefined;
        schemas?: Record<string, Record<string, any>> | undefined;
        requestBodies?: Record<string, {
            content: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }>;
            description?: string | undefined;
            required?: boolean | undefined;
            encoding?: Record<string, {
                contentType: string;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
            }> | undefined;
        }> | undefined;
        securitySchemes?: Record<string, {
            type: "apiKey";
            description?: string | undefined;
            name?: string | undefined;
            in?: unknown;
        } | {
            type: "http";
            description?: string | undefined;
            scheme?: string | undefined;
            bearerFormat?: string | undefined;
        } | {
            type: "openIdConnect";
            description?: string | undefined;
            openIdConnectUrl?: string | undefined;
        } | {
            type: "oauth2";
            flows: {
                implicit?: {
                    type?: "implicit" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    authorizationUrl?: string | undefined;
                } | undefined;
                password?: {
                    type?: "password" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    tokenUrl?: string | undefined;
                } | undefined;
                clientCredentials?: {
                    type?: "clientCredentials" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    tokenUrl?: string | undefined;
                } | undefined;
                authorizationCode?: {
                    type?: "authorizationCode" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    authorizationUrl?: string | undefined;
                    tokenUrl?: string | undefined;
                } | undefined;
            };
            description?: string | undefined;
        } | {
            type: "mutualTLS";
            description?: string | undefined;
        }> | undefined;
        pathItems?: Record<string, {
            servers?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            }[] | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            get?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            put?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            post?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            delete?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            options?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            head?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            patch?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            trace?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
        }> | undefined;
    }>>;
    /**
     * A declaration of which security mechanisms can be used across the API. The list of values includes alternative
     * Security Requirement Objects that can be used. Only one of the Security Requirement Objects need to be satisfied
     * to authorize a request. Individual operations can override this definition. The list can be incomplete, up to
     * being empty or absent. To make security explicitly optional, an empty security requirement ({}) can be included
     * in the array.
     */
    security: z.ZodOptional<z.ZodArray<z.ZodRecord<z.ZodString, z.ZodArray<z.ZodString, "many">>, "many">>;
    /**
     * A list of tags used by the OpenAPI Description with additional metadata. The order of the tags can be used to
     * reflect on their order by the parsing tools. Not all tags that are used by the Operation Object must be declared.
     * The tags that are not declared MAY be organized randomly or based on the tools' logic. Each tag name in the list
     * MUST be unique.
     */
    tags: z.ZodOptional<z.ZodArray<z.ZodObject<{
        name: z.ZodString;
        description: z.ZodCatch<z.ZodOptional<z.ZodString>>;
        externalDocs: z.ZodOptional<z.ZodObject<{
            description: z.ZodOptional<z.ZodString>;
            url: z.ZodString;
        }, "strip", z.ZodTypeAny, {
            url: string;
            description?: string | undefined;
        }, {
            url: string;
            description?: string | undefined;
        }>>;
    }, "strip", z.ZodTypeAny, {
        name: string;
        externalDocs?: {
            url: string;
            description?: string | undefined;
        } | undefined;
        description?: string | undefined;
    }, {
        name: string;
        externalDocs?: {
            url: string;
            description?: string | undefined;
        } | undefined;
        description?: unknown;
    }>, "many">>;
    /**
     * Additional external documentation.
     */
    externalDocs: z.ZodOptional<z.ZodObject<{
        description: z.ZodOptional<z.ZodString>;
        url: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        url: string;
        description?: string | undefined;
    }, {
        url: string;
        description?: string | undefined;
    }>>;
}, "strip", z.ZodTypeAny, {
    openapi: string;
    info: {
        title: string;
        version: string;
        description?: string | undefined;
        termsOfService?: string | undefined;
        contact?: {
            url?: string | undefined;
            name?: string | undefined;
            email?: string | undefined;
        } | undefined;
        license?: {
            url?: string | undefined;
            name?: string | null | undefined;
            identifier?: string | undefined;
        } | undefined;
        summary?: string | undefined;
    };
    paths?: Record<string, {
        servers?: {
            url: string;
            description?: string | undefined;
            variables?: Record<string, {
                description?: string | undefined;
                default?: string | undefined;
                enum?: string[] | undefined;
            }> | undefined;
        }[] | undefined;
        description?: string | undefined;
        summary?: string | undefined;
        parameters?: {
            name: string;
            in: "path" | "query" | "header" | "cookie";
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
            allowEmptyValue?: boolean | undefined;
            allowReserved?: boolean | undefined;
        }[] | undefined;
        get?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        put?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        post?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        delete?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        options?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        head?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        patch?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        trace?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
    }> | undefined;
    components?: {
        parameters?: Record<string, {
            name: string;
            in: "path" | "query" | "header" | "cookie";
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
            allowEmptyValue?: boolean | undefined;
            allowReserved?: boolean | undefined;
        }> | undefined;
        responses?: Record<string, {
            description: string;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            headers?: Record<string, {
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
            }> | undefined;
            links?: Record<string, {
                description?: string | undefined;
                operationId?: string | undefined;
                parameters?: Record<string, string> | undefined;
                requestBody?: string | undefined;
                operationRef?: string | undefined;
                server?: {
                    url: string;
                    description?: string | undefined;
                    variables?: Record<string, {
                        description?: string | undefined;
                        default?: string | undefined;
                        enum?: string[] | undefined;
                    }> | undefined;
                } | undefined;
            }> | undefined;
        }> | undefined;
        callbacks?: Record<string, Record<string, {
            servers?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            }[] | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
        } & {
            get?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            put?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            post?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            delete?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            options?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            head?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            patch?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            trace?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        }>> | undefined;
        headers?: Record<string, {
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
        }> | undefined;
        links?: Record<string, {
            description?: string | undefined;
            operationId?: string | undefined;
            parameters?: Record<string, string> | undefined;
            requestBody?: string | undefined;
            operationRef?: string | undefined;
            server?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            } | undefined;
        }> | undefined;
        examples?: Record<string, {
            description?: string | undefined;
            summary?: string | undefined;
            value?: any;
            externalValue?: string | undefined;
        }> | undefined;
        schemas?: Record<string, Record<string, any>> | undefined;
        requestBodies?: Record<string, {
            content: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }>;
            description?: string | undefined;
            required?: boolean | undefined;
            encoding?: Record<string, {
                contentType: string;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
            }> | undefined;
        }> | undefined;
        securitySchemes?: Record<string, {
            name: string;
            in: "query" | "header" | "cookie";
            type: "apiKey";
            description?: string | undefined;
        } | {
            type: "http";
            scheme: "basic" | "bearer";
            description?: string | undefined;
            bearerFormat?: string | undefined;
        } | {
            type: "openIdConnect";
            openIdConnectUrl: string;
            description?: string | undefined;
        } | {
            type: "oauth2";
            flows: {
                implicit?: {
                    scopes: Record<string, string | undefined>;
                    authorizationUrl: string;
                    type?: "implicit" | undefined;
                    refreshUrl?: string | undefined;
                } | undefined;
                password?: {
                    scopes: Record<string, string | undefined>;
                    tokenUrl: string;
                    type?: "password" | undefined;
                    refreshUrl?: string | undefined;
                } | undefined;
                clientCredentials?: {
                    scopes: Record<string, string | undefined>;
                    tokenUrl: string;
                    type?: "clientCredentials" | undefined;
                    refreshUrl?: string | undefined;
                } | undefined;
                authorizationCode?: {
                    scopes: Record<string, string | undefined>;
                    authorizationUrl: string;
                    tokenUrl: string;
                    type?: "authorizationCode" | undefined;
                    refreshUrl?: string | undefined;
                } | undefined;
            };
            description?: string | undefined;
        } | {
            type: "mutualTLS";
            description?: string | undefined;
        }> | undefined;
        pathItems?: Record<string, {
            servers?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            }[] | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            get?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            put?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            post?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            delete?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            options?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            head?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            patch?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            trace?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
        }> | undefined;
    } | undefined;
    servers?: {
        url: string;
        description?: string | undefined;
        variables?: Record<string, {
            description?: string | undefined;
            default?: string | undefined;
            enum?: string[] | undefined;
        }> | undefined;
    }[] | undefined;
    security?: Record<string, string[]>[] | undefined;
    tags?: {
        name: string;
        externalDocs?: {
            url: string;
            description?: string | undefined;
        } | undefined;
        description?: string | undefined;
    }[] | undefined;
    externalDocs?: {
        url: string;
        description?: string | undefined;
    } | undefined;
    webhooks?: Record<string, {
        servers?: {
            url: string;
            description?: string | undefined;
            variables?: Record<string, {
                description?: string | undefined;
                default?: string | undefined;
                enum?: string[] | undefined;
            }> | undefined;
        }[] | undefined;
        description?: string | undefined;
        summary?: string | undefined;
        parameters?: {
            name: string;
            in: "path" | "query" | "header" | "cookie";
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
            allowEmptyValue?: boolean | undefined;
            allowReserved?: boolean | undefined;
        }[] | undefined;
    } & {
        get?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        put?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        post?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        delete?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        options?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        head?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        patch?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        trace?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
    }> | undefined;
    jsonSchemaDialect?: string | undefined;
}, {
    openapi: string;
    info: {
        title?: unknown;
        description?: unknown;
        termsOfService?: unknown;
        contact?: unknown;
        license?: unknown;
        version?: unknown;
        summary?: unknown;
    };
    paths?: Record<string, {
        servers?: {
            url: string;
            description?: string | undefined;
            variables?: Record<string, {
                description?: string | undefined;
                default?: string | undefined;
                enum?: string[] | undefined;
            }> | undefined;
        }[] | undefined;
        description?: string | undefined;
        summary?: string | undefined;
        parameters?: {
            name: string;
            in: "path" | "query" | "header" | "cookie";
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
            allowEmptyValue?: boolean | undefined;
            allowReserved?: boolean | undefined;
        }[] | undefined;
        get?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        put?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        post?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        delete?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        options?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        head?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        patch?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
        trace?: ({
            security?: Record<string, string[]>[] | undefined;
            tags?: string[] | undefined;
            externalDocs?: {
                url: string;
                description?: string | undefined;
            } | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            operationId?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            requestBody?: {
                content: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }>;
                description?: string | undefined;
                required?: boolean | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            } | undefined;
            responses?: Record<string, {
                description: string;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
                links?: Record<string, {
                    description?: string | undefined;
                    operationId?: string | undefined;
                    parameters?: Record<string, string> | undefined;
                    requestBody?: string | undefined;
                    operationRef?: string | undefined;
                    server?: {
                        url: string;
                        description?: string | undefined;
                        variables?: Record<string, {
                            description?: string | undefined;
                            default?: string | undefined;
                            enum?: string[] | undefined;
                        }> | undefined;
                    } | undefined;
                }> | undefined;
            }> | undefined;
            deprecated?: boolean | undefined;
        } & {
            callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
        }) | undefined;
    }> | undefined;
    components?: {
        parameters?: Record<string, {
            name: string;
            in: "path" | "query" | "header" | "cookie";
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
            allowEmptyValue?: boolean | undefined;
            allowReserved?: boolean | undefined;
        }> | undefined;
        responses?: Record<string, {
            description: string;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            headers?: Record<string, {
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
            }> | undefined;
            links?: Record<string, {
                description?: string | undefined;
                operationId?: string | undefined;
                parameters?: Record<string, string> | undefined;
                requestBody?: string | undefined;
                operationRef?: string | undefined;
                server?: {
                    url: string;
                    description?: string | undefined;
                    variables?: Record<string, {
                        description?: string | undefined;
                        default?: string | undefined;
                        enum?: string[] | undefined;
                    }> | undefined;
                } | undefined;
            }> | undefined;
        }> | undefined;
        callbacks?: Record<string, Record<string, {
            servers?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            }[] | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
        } & {
            get?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            put?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            post?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            delete?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            options?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            head?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            patch?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
            trace?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        }>> | undefined;
        headers?: Record<string, {
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
        }> | undefined;
        links?: Record<string, {
            description?: string | undefined;
            operationId?: string | undefined;
            parameters?: Record<string, string> | undefined;
            requestBody?: string | undefined;
            operationRef?: string | undefined;
            server?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            } | undefined;
        }> | undefined;
        examples?: Record<string, {
            description?: string | undefined;
            summary?: string | undefined;
            value?: any;
            externalValue?: string | undefined;
        }> | undefined;
        schemas?: Record<string, Record<string, any>> | undefined;
        requestBodies?: Record<string, {
            content: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }>;
            description?: string | undefined;
            required?: boolean | undefined;
            encoding?: Record<string, {
                contentType: string;
                headers?: Record<string, {
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                }> | undefined;
            }> | undefined;
        }> | undefined;
        securitySchemes?: Record<string, {
            type: "apiKey";
            description?: string | undefined;
            name?: string | undefined;
            in?: unknown;
        } | {
            type: "http";
            description?: string | undefined;
            scheme?: string | undefined;
            bearerFormat?: string | undefined;
        } | {
            type: "openIdConnect";
            description?: string | undefined;
            openIdConnectUrl?: string | undefined;
        } | {
            type: "oauth2";
            flows: {
                implicit?: {
                    type?: "implicit" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    authorizationUrl?: string | undefined;
                } | undefined;
                password?: {
                    type?: "password" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    tokenUrl?: string | undefined;
                } | undefined;
                clientCredentials?: {
                    type?: "clientCredentials" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    tokenUrl?: string | undefined;
                } | undefined;
                authorizationCode?: {
                    type?: "authorizationCode" | undefined;
                    refreshUrl?: string | undefined;
                    scopes?: unknown;
                    authorizationUrl?: string | undefined;
                    tokenUrl?: string | undefined;
                } | undefined;
            };
            description?: string | undefined;
        } | {
            type: "mutualTLS";
            description?: string | undefined;
        }> | undefined;
        pathItems?: Record<string, {
            servers?: {
                url: string;
                description?: string | undefined;
                variables?: Record<string, {
                    description?: string | undefined;
                    default?: string | undefined;
                    enum?: string[] | undefined;
                }> | undefined;
            }[] | undefined;
            description?: string | undefined;
            summary?: string | undefined;
            parameters?: {
                name: string;
                in: "path" | "query" | "header" | "cookie";
                description?: string | undefined;
                deprecated?: boolean | undefined;
                content?: Record<string, {
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                }> | undefined;
                required?: boolean | undefined;
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                explode?: boolean | undefined;
                allowEmptyValue?: boolean | undefined;
                allowReserved?: boolean | undefined;
            }[] | undefined;
            get?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            put?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            post?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            delete?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            options?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            head?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            patch?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
            trace?: ({
                security?: Record<string, string[]>[] | undefined;
                tags?: string[] | undefined;
                externalDocs?: {
                    url: string;
                    description?: string | undefined;
                } | undefined;
                description?: string | undefined;
                summary?: string | undefined;
                operationId?: string | undefined;
                parameters?: {
                    name: string;
                    in: "path" | "query" | "header" | "cookie";
                    description?: string | undefined;
                    deprecated?: boolean | undefined;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    required?: boolean | undefined;
                    example?: any;
                    examples?: Record<string, {
                        description?: string | undefined;
                        summary?: string | undefined;
                        value?: any;
                        externalValue?: string | undefined;
                    }> | undefined;
                    schema?: Record<string, any> | undefined;
                    style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                    explode?: boolean | undefined;
                    allowEmptyValue?: boolean | undefined;
                    allowReserved?: boolean | undefined;
                }[] | undefined;
                requestBody?: {
                    content: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }>;
                    description?: string | undefined;
                    required?: boolean | undefined;
                    encoding?: Record<string, {
                        contentType: string;
                        headers?: Record<string, {
                            description?: string | undefined;
                            deprecated?: boolean | undefined;
                            content?: Record<string, {
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                            }> | undefined;
                            required?: boolean | undefined;
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                            explode?: boolean | undefined;
                        }> | undefined;
                    }> | undefined;
                } | undefined;
                responses?: Record<string, {
                    description: string;
                    content?: Record<string, {
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        encoding?: Record<string, {
                            contentType: string;
                            headers?: Record<string, {
                                description?: string | undefined;
                                deprecated?: boolean | undefined;
                                content?: Record<string, {
                                    example?: any;
                                    examples?: Record<string, {
                                        description?: string | undefined;
                                        summary?: string | undefined;
                                        value?: any;
                                        externalValue?: string | undefined;
                                    }> | undefined;
                                    schema?: Record<string, any> | undefined;
                                }> | undefined;
                                required?: boolean | undefined;
                                example?: any;
                                examples?: Record<string, {
                                    description?: string | undefined;
                                    summary?: string | undefined;
                                    value?: any;
                                    externalValue?: string | undefined;
                                }> | undefined;
                                schema?: Record<string, any> | undefined;
                                style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                                explode?: boolean | undefined;
                            }> | undefined;
                        }> | undefined;
                    }> | undefined;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                    links?: Record<string, {
                        description?: string | undefined;
                        operationId?: string | undefined;
                        parameters?: Record<string, string> | undefined;
                        requestBody?: string | undefined;
                        operationRef?: string | undefined;
                        server?: {
                            url: string;
                            description?: string | undefined;
                            variables?: Record<string, {
                                description?: string | undefined;
                                default?: string | undefined;
                                enum?: string[] | undefined;
                            }> | undefined;
                        } | undefined;
                    }> | undefined;
                }> | undefined;
                deprecated?: boolean | undefined;
            } & {
                callbacks?: Record<string, z.infer<typeof import("./callback-object.js").CallbackObjectSchema>>;
            }) | undefined;
        }> | undefined;
    } | undefined;
    servers?: {
        url: string;
        description?: string | undefined;
        variables?: Record<string, {
            description?: string | undefined;
            default?: string | undefined;
            enum?: string[] | undefined;
        }> | undefined;
    }[] | undefined;
    security?: Record<string, string[]>[] | undefined;
    tags?: {
        name: string;
        externalDocs?: {
            url: string;
            description?: string | undefined;
        } | undefined;
        description?: unknown;
    }[] | undefined;
    externalDocs?: {
        url: string;
        description?: string | undefined;
    } | undefined;
    webhooks?: Record<string, {
        servers?: {
            url: string;
            description?: string | undefined;
            variables?: Record<string, {
                description?: string | undefined;
                default?: string | undefined;
                enum?: string[] | undefined;
            }> | undefined;
        }[] | undefined;
        description?: string | undefined;
        summary?: string | undefined;
        parameters?: {
            name: string;
            in: "path" | "query" | "header" | "cookie";
            description?: string | undefined;
            deprecated?: boolean | undefined;
            content?: Record<string, {
                example?: any;
                examples?: Record<string, {
                    description?: string | undefined;
                    summary?: string | undefined;
                    value?: any;
                    externalValue?: string | undefined;
                }> | undefined;
                schema?: Record<string, any> | undefined;
                encoding?: Record<string, {
                    contentType: string;
                    headers?: Record<string, {
                        description?: string | undefined;
                        deprecated?: boolean | undefined;
                        content?: Record<string, {
                            example?: any;
                            examples?: Record<string, {
                                description?: string | undefined;
                                summary?: string | undefined;
                                value?: any;
                                externalValue?: string | undefined;
                            }> | undefined;
                            schema?: Record<string, any> | undefined;
                        }> | undefined;
                        required?: boolean | undefined;
                        example?: any;
                        examples?: Record<string, {
                            description?: string | undefined;
                            summary?: string | undefined;
                            value?: any;
                            externalValue?: string | undefined;
                        }> | undefined;
                        schema?: Record<string, any> | undefined;
                        style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
                        explode?: boolean | undefined;
                    }> | undefined;
                }> | undefined;
            }> | undefined;
            required?: boolean | undefined;
            example?: any;
            examples?: Record<string, {
                description?: string | undefined;
                summary?: string | undefined;
                value?: any;
                externalValue?: string | undefined;
            }> | undefined;
            schema?: Record<string, any> | undefined;
            style?: "matrix" | "label" | "simple" | "form" | "spaceDelimited" | "pipeDelimited" | "deepObject" | undefined;
            explode?: boolean | undefined;
            allowEmptyValue?: boolean | undefined;
            allowReserved?: boolean | undefined;
        }[] | undefined;
    } & {
        get?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        put?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        post?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        delete?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        options?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        head?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        patch?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
        trace?: z.infer<typeof import("./operation-object-without-callbacks.js").OperationObjectSchemaWithoutCallbacks>;
    }> | undefined;
    jsonSchemaDialect?: string | undefined;
}>;
//# sourceMappingURL=openapi-object.d.ts.map